@@ -1,16 +1,21 @@
 package AI::ConfusionMatrix;
-$AI::ConfusionMatrix::VERSION = '0.006';
+$AI::ConfusionMatrix::VERSION = '0.007';
+use strict;
+use warnings;
 use Carp;
 use Exporter 'import';
 our @EXPORT= qw (makeConfusionMatrix);
 use strict;
 use Tie::File;
-use warnings;
 
 # ABSTRACT: Make a confusion matrix
 
 sub makeConfusionMatrix {
-    my ($matrix, $file) = @_;
+    my ($matrix, $file, $delem) = @_;
+    unless(defined $delem) {
+        $delem = ',';
+    }
+
     carp ('First argument must be a hash reference') if ref($matrix) ne 'HASH';
     tie my @array, 'Tie::File', $file or carp "$!";
     my $n = 1;
@@ -39,7 +44,7 @@ sub makeConfusionMatrix {
             push @columns, $predicted unless _findIndex($predicted, \@columns);
         }
 
-        $stats{$expected}{'acc'} = sprintf("%.2f", ($stats{$expected}{'tp'} * 100) / $stats{$expected}{'total'});
+        $stats{$expected}{'acc'} = sprintf("%.2f%%", ($stats{$expected}{'tp'} * 100) / $stats{$expected}{'total'});
     }
 
     for my $expected (@expected) {
@@ -47,12 +52,13 @@ sub makeConfusionMatrix {
         $totals{'tp'}    += $stats{$expected}{'tp'};
         $totals{'fn'}    += $stats{$expected}{'fn'};
         $totals{'fp'}    += $stats{$expected}{'fp'};
-        $stats{$expected}{'acc'} .= '%';
+        $stats{$expected}{'sensitivity'} = sprintf("%.2f%%", (($stats{$expected}{'tp'} * 100) / ($stats{$expected}{'tp'} + $stats{$expected}{'fp'})));
     }
 
     $totals{'acc'} = sprintf("%.2f%%", ($totals{'tp'} * 100) / $totals{'total'});
+    $totals{'sensitivity'} = sprintf("%.2f%%", ($totals{'tp'} * 100) / ($totals{'tp'} + $totals{'fp'}));
     @columns = sort @columns;
-    map {$array[0] .= ',' . $_} join ',', (@columns, 'TOTAL', 'TP', 'FP', 'FN', 'ACC');
+    map {$array[0] .= $delem . $_} join $delem, (@columns, 'TOTAL', 'TP', 'FP', 'FN', 'SENS', 'ACC');
     $n = 1;
     for my $expected (@expected) {
         my $lastIndex = 0;
@@ -60,26 +66,27 @@ sub makeConfusionMatrix {
         for my $predicted (sort keys %{$matrix->{$expected}}) {
             # Calculate the index of the label in the array of columns
             $index = _findIndex($predicted, \@columns);
-            # Print some commas to get to the column of the next value predicted
-            $array[$n] .= ',' x ($index - $lastIndex) . $matrix->{$expected}{$predicted};
+            # Print some of the delimiter to get to the column of the next value predicted
+            $array[$n] .= $delem x ($index - $lastIndex) . $matrix->{$expected}{$predicted};
             $lastIndex = $index;
         }
 
         # Get to the columns of the stats
-        $array[$n] .= ',' x (scalar(@columns) - $lastIndex + 1);
-        $array[$n] .= join ',', (
+        $array[$n] .= $delem x (scalar(@columns) - $lastIndex + 1);
+        $array[$n] .= join $delem, (
             $stats{$expected}{'total'},
             $stats{$expected}{'tp'},
             $stats{$expected}{'fp'},
             $stats{$expected}{'fn'},
+            $stats{$expected}{'sensitivity'},
             $stats{$expected}{'acc'}
         );
         ++$n;
     }
     # Print the TOTAL row to the csv file
-    $array[$n] = 'TOTAL,';
-    map {$array[$n] .= $totals{$_}. ','} (sort keys %totals)[0 .. $#columns];
-    $array[$n] .= join ',', ($totals{'total'}, $totals{'tp'}, $totals{'fp'}, $totals{'fn'}, $totals{'acc'});
+    $array[$n] = 'TOTAL' . $delem;
+    map {$array[$n] .= $totals{$_} . $delem} (sort keys %totals)[0 .. $#columns];
+    $array[$n] .= join $delem, ($totals{'total'}, $totals{'tp'}, $totals{'fp'}, $totals{'fn'}, $totals{'sensitivity'}, $totals{'acc'});
 
     untie @array;
 }
@@ -116,13 +123,14 @@ This module prints a L<confusion matrix|https://en.wikipedia.org/wiki/Confusion_
 
 =head3 Function
 
-=head4 C<makeConfusionMatrix($hash_ref, $file)>
+=head4 C<makeConfusionMatrix($hash_ref, $file [, $delimiter ])>
 
-This function makes a confusion matrix from C<$hash_ref> and writes it to C<$file>. C<$file> can be a filename or a file handle opened with the C<w+> mode.
+This function makes a confusion matrix from C<$hash_ref> and writes it to C<$file>. C<$file> can be a filename or a file handle opened with the C<w+> mode. If C<$delimiter> is present, it is used as a custom separator for the fields in the confusion matrix.
 
 Examples:
 
     makeConfusionMatrix(\%matrix, 'output.csv');
+    makeConfusionMatrix(\%matrix, 'output.csv', ';');
     makeConfusionMatrix(\%matrix, *$fh);
 
 The hash reference must look like this :
@@ -145,31 +153,24 @@ The hash reference must look like this :
 
 The output will be in CSV. Here is an example:
 
-
-    ,1997,1998,2001,2003,2005,2008,2012,2015,TOTAL,TP,FP,FN,ACC
-    1997,2,,,,,,,,2,2,0,0,100.00%
-    1998,,1,,,,,,,1,1,0,0,100.00%
-    2001,,,1,,,,,,1,1,0,0,100.00%
-    2003,,,,5,,,1,1,7,5,0,2,71.43%
-    2005,,,,,7,,,2,9,7,0,2,77.78%
-    2008,,,,,,3,,,3,3,0,0,100.00%
-    2012,,,,,,,5,,5,5,1,0,100.00%
-    2015,,,,,,,,8,8,8,3,0,100.00%
-    TOTAL,2,1,1,5,7,3,6,11,36,32,4,4,88.89%
+    ,1974,1978,2002,2003,2005,TOTAL,TP,FP,FN,SENS,ACC
+    1974,3,1,,,2,6,3,4,3,42.86%,50.00%
+    1978,1,5,,,,6,5,4,1,55.56%,83.33%
+    2002,2,2,8,,,12,8,1,4,88.89%,66.67%
+    2003,1,,,7,2,10,7,0,3,100.00%,70.00%
+    2005,,1,1,,6,8,6,4,2,60.00%,75.00%
+    TOTAL,7,9,9,7,10,42,29,13,13,69.05%,69.05%
 
 Prettified:
 
-    |       | 1997 | 1998 | 2001 | 2003 | 2005 | 2008 | 2012 | 2015 | TOTAL | TP | FP | FN | ACC     |
-    |-------|------|------|------|------|------|------|------|------|-------|----|----|----|---------|
-    | 1997  | 2    |      |      |      |      |      |      |      | 2     | 2  | 0  | 0  | 100.00% |
-    | 1998  |      | 1    |      |      |      |      |      |      | 1     | 1  | 0  | 0  | 100.00% |
-    | 2001  |      |      | 1    |      |      |      |      |      | 1     | 1  | 0  | 0  | 100.00% |
-    | 2003  |      |      |      | 5    |      |      | 1    | 1    | 7     | 5  | 0  | 2  | 71.43%  |
-    | 2005  |      |      |      |      | 7    |      |      | 2    | 9     | 7  | 0  | 2  | 77.78%  |
-    | 2008  |      |      |      |      |      | 3    |      |      | 3     | 3  | 0  | 0  | 100.00% |
-    | 2012  |      |      |      |      |      |      | 5    |      | 5     | 5  | 1  | 0  | 100.00% |
-    | 2015  |      |      |      |      |      |      |      | 8    | 8     | 8  | 3  | 0  | 100.00% |
-    | TOTAL | 2    | 1    | 1    | 5    | 7    | 3    | 6    | 11   | 36    | 32 | 4  | 4  | 88.89%  |
+    |       | 1974 | 1978 | 2002 | 2003 | 2005 | TOTAL | TP | FP | FN | SENS    | ACC    |
+    |-------|------|------|------|------|------|-------|----|----|----|---------|--------|
+    | 1974  | 3    | 1    |      |      | 2    | 6     | 3  | 4  | 3  | 42.86%  | 50.00% |
+    | 1978  | 1    | 5    |      |      |      | 6     | 5  | 4  | 1  | 55.56%  | 83.33% |
+    | 2002  | 2    | 2    | 8    |      |      | 12    | 8  | 1  | 4  | 88.89%  | 66.67% |
+    | 2003  | 1    |      |      | 7    | 2    | 10    | 7  | 0  | 3  | 100.00% | 70.00% |
+    | 2005  |      | 1    | 1    |      | 6    | 8     | 6  | 4  | 2  | 60.00%  | 75.00% |
+    | TOTAL | 7    | 9    | 9    | 7    | 10   | 42    | 29 | 13 | 13 | 69.05%  | 69.05% |
 
 =over
 
@@ -185,6 +186,10 @@ False Positive
 
 False Negative
 
+=item SENS
+
+Sensitivity. Number of true positives divided by the number of positives.
+
 =item ACC:
 
 Accuracy
@@ -193,7 +198,7 @@ Accuracy
 
 =head1 AUTHOR
 
-Vincent Lequertier <sky@riseup.net>
+Vincent Lequertier <vi.le@autistici.org>
 
 =head1 LICENSE
 
